/*	Renegade Scripts.dll
    ArKa_47 Spy Chat Plugin
	Copyright 2020 ArKa_47, Shadow-Team

	This file is part of the Renegade scripts.dll
	The Renegade scripts.dll is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.
	In addition, an exemption is given to allow Run Time Dynamic Linking of this code with any closed source module that does not contain code covered by this licence.
	Only the source code to the module(s) containing the licenced code has to be released.
*/

#include "general.h"
#include "engine.h"
#include "engine_da.h"
#include "da_crate.h"
#include "da.h"
#include "da_event.h"
#include "da_chatcommand.h"
#include "da_player.h"
#include "da_vehicle.h"
#include "GameObjManager.h"

#include "SpyChat.h"

// Create the Plugin
ArKaSpyChatClass SpyChatPlugIn;

// ---------------------------- AKSpyChatClass constructor ----------------------------------------------------------------------
ArKaSpyChatClass::ArKaSpyChatClass() {
	Console_Output("Loading Spy crate modification [beta] ; written by ArKa_47\n");

	// initalise the whole array false
	for (int i = 0; i <= MAX_SPY; i++)
		SpyList[i] = false;
	SpyCount = 0;

	Register_Event(DAEvent::CHAT, INT_MAX);
	Register_Event(DAEvent::GAMEOVER, INT_MAX);
}

// ---------------------------- AKSpyChatClass destructor -----------------------------------------------------------------------
ArKaSpyChatClass::~ArKaSpyChatClass() {
	Console_Output("Un-Loading Spy crate modification [beta] ; written by ArKa_47\n");

	Unregister_Event(DAEvent::CHAT);
	Unregister_Event(DAEvent::GAMEOVER);
}

// ---------------------------- Reset the SpyList and the counter on Game Over --------------------------------------------------
void ArKaSpyChatClass::Game_Over_Event() {

	// Set all spies as false and reset the SpyCounter
	for (int i = 1; i <= MAX_PLAYERS; i++)
		SpyList[i] = false;
	SpyCount = 0;
}

// ---------------------------- Set Player ID in SpyList as TRUE and start the SpyCreated scenario ------------------------------
void ArKaSpyChatClass::Add_To_Spylist(int PlayerId) {
	if (PlayerId >= 1 && PlayerId <= 128) {
		SpyList[PlayerId] = true;
		Start_Timer(10, 2.0f, false, (unsigned int)PlayerId);
	}
}

// ---------------------------- Set Player ID in SpyList as FALSE and end the SpyCreated scenario -------------------------------
void ArKaSpyChatClass::Delete_From_Spylist(int PlayerId) {
	if (PlayerId >= 1 && PlayerId <= 128) {
		SpyList[PlayerId] = false;
		Start_Timer(12, 1.8f, false, (unsigned int)PlayerId);
	}
}

// ---------------------------- Scan all players for spy ability ----------------------------------------------------------------
void ArKaSpyChatClass::ScanSpies() {
	for (int PlayerID = 1; PlayerID <= MAX_PLAYERS; PlayerID++) {
		if (Is_Player(Get_GameObj(PlayerID))) {
			if (!Get_Is_Visible(PlayerID))
				Add_To_Spylist(PlayerID);
		}
	}
}

// ---------------------------- Check if a player is visible to enemie's base defense ------------------------------------------
bool ArKaSpyChatClass::Get_Is_Visible(int PlayerId){
	if (PlayerId > 0 && PlayerId <= MAX_PLAYERS) {
		GameObject* obj = Get_GameObj(PlayerId);
		if (Is_Player(obj))
			return obj->As_SoldierGameObj()->Is_Visible();
		else
			return true;
	}
	else return true;
}

bool ArKaSpyChatClass::Get_Is_Visible(GameObject* obj){
		if (Is_Player(obj))
			return obj->As_SoldierGameObj()->Is_Visible();
		else
			return false;
}

// --------------------- S P Y   R E A D S   E N E M I E ' S   T E A M   C H A T ------------------------------------------------
bool ArKaSpyChatClass::Chat_Event(cPlayer* Sender, TextMessageEnum Type, const wchar_t* Message, int ReceiverID) {
	if (SpyCount > 0) {
		if (Type == TEXT_MESSAGE_TEAM){
			for (int PlayerID = 1; PlayerID <= MAX_PLAYERS; PlayerID++) {
				if (SpyList[PlayerID] && Is_Player(Get_GameObj(PlayerID))) {
					if (Get_Team(PlayerID) != Get_Team(Sender->Get_Id())) {
						DA::Private_Color_Message_With_Team_Color(Get_GameObj(PlayerID), Get_Team(Sender->Get_Id()), "%ls: %ls", Sender->Get_Name(), Message);
					}
				}
			}
		}
	}
	return true;
}

// ---------------------------- Timer for spy created scenario ------------------------------------------------------------------
void ArKaSpyChatClass::Timer_Expired(int Number, unsigned int PlayerID) {
	if (Number == 1) {

	//	ScanSpies();
	} else if (Number == 10) {
		if (Is_Player(Get_GameObj(PlayerID))){
			if (Get_Team(PlayerID)==PLAYERTYPE_GDI)
				Create_2D_WAV_Sound_Player(Get_GameObj(PlayerID), "M00EVAG_DSGN0031I1EVAG_snd.wav");
			else
				Create_2D_WAV_Sound_Player(Get_GameObj(PlayerID), "M00EVAN_DSGN0027I1EVAN_snd.wav");

			Send_Message_Player(Get_GameObj(PlayerID), COLORWHITE, "Uplink established.");
			Start_Timer(11, 1.0f, false, (unsigned int)PlayerID);
		}
	}
	else if (Number == 11) {
		if (Is_Player(Get_GameObj(PlayerID))) {
			if (Get_Team(PlayerID) == PLAYERTYPE_GDI)
				Create_2D_WAV_Sound_Player(Get_GameObj(PlayerID), "M00EVAG_DSGN0042i1EVAG_snd.wav");
			else
				Create_2D_WAV_Sound_Player(Get_GameObj(PlayerID), "M00EVAN_DSGN0038i1EVAN_snd.wav");

			Send_Message_Player(Get_GameObj(PlayerID), COLORWHITE, "Unauthorized incoming data transmission...");
			//Start_Timer(13, 2.0f, false, (unsigned int)PlayerID);
		}
	}
	else if (Number == 12) {
		if (Is_Player(Get_GameObj(PlayerID))) {
			if (Get_Team(PlayerID) == PLAYERTYPE_GDI) {
				Create_2D_WAV_Sound_Player(Get_GameObj(PlayerID), "M00EVAG_DSGN0036I1EVAG_snd.wav");
				Send_Message_Player(Get_GameObj(PlayerID), COLORWHITE, "Reconnection failed, you can not longer spy NOD's communication.");
			}
			else{
				Create_2D_WAV_Sound_Player(Get_GameObj(PlayerID), "M00EVAN_DSGN0032I1EVAN_snd.wav");
			Send_Message_Player(Get_GameObj(PlayerID), COLORWHITE, "Reconnection failed, you can not longer spy GDI's communication.");
			}
		}
	}
	else if (Number == 13) {
		if (Is_Player(Get_GameObj(PlayerID))) {
			if (Get_Team(PlayerID) == PLAYERTYPE_GDI)
			Send_Message_Player(Get_GameObj(PlayerID), COLORWHITE, "You can now spy NOD's communication.");
			else
				Send_Message_Player(Get_GameObj(PlayerID), COLORWHITE, "You can now spy GDI's communication.");
		}
	}
}


/*************************** ADDED SPY_AND_CHAT CRATE ******************************************************************************/


// ------------------------- single SpyChatOberverClass to let the Player spy in to the enemie's teamchat -----------------------------------
class ArKaSpyChatObserverClass : public DAGameObjObserverClass {
public:
	// duration in seconds , 60 by default
	float time = 60;

	void Set_Time(float time) {
		this->time = time;
	}
	float Get_Time() {
		return time;
	}

	virtual const char* Get_Name() {
		return "ArKaSpyChatObserverClass";
	}

	virtual void Init() {
		SpyChatPlugIn.Add_To_Spylist(Get_Player_ID(Get_Owner()));
		SpyChatPlugIn.SpyCount++;
		Start_Timer(100, time);
	}


	virtual void Destroyed(GameObject* obj) {
		SpyChatPlugIn.Delete_From_Spylist(Get_Player_ID(obj));
		Set_Delete_Pending();
	}


	virtual void Kill_Dealt(DamageableGameObj* Victim, float Damage, unsigned int Warhead, float Scale, DADamageType::Type Type) {
		if (Victim->As_BuildingGameObj()) {
			ArKaSpyChatObserverClass::Set_Delete_Pending();
			SpyChatPlugIn.Delete_From_Spylist(Get_Player_ID(Get_Owner()));

			Set_Delete_Pending();
		}
	}

	virtual void Timer_Expired(GameObject* obj, int Number) {

		if (Number == 100) {
			SpyChatPlugIn.Delete_From_Spylist(Get_Player_ID(Get_Owner()));
			Set_Delete_Pending();

		}
	}

	~ArKaSpyChatObserverClass() {
		SpyChatPlugIn.SpyCount--;


	}
};

// -----------------------------------------------------------------------------------------------------------------------

class ArKaSpyObserverClass : public DAGameObjObserverClass {
	


	virtual const char* Get_Name() {
		return "ArKaSpyObserverClass";
	}


	virtual void Init() {
		((SoldierGameObj*)Get_Owner())->Set_Is_Visible(false);

	}


	virtual void Kill_Dealt(DamageableGameObj* Victim, float Damage, unsigned int Warhead, float Scale, DADamageType::Type Type) {
		if (Victim->As_BuildingGameObj()) {

			Set_Delete_Pending();
		}
	}

	~ArKaSpyObserverClass() {
		((SoldierGameObj*)Get_Owner())->Set_Is_Visible(true);

	}
};

class ArKaSpyCrateClass : public DACrateClass {
	virtual bool Can_Activate(cPlayer* Player) { //Prevent crate from triggering if enemy base defense is dead or powered down.
		BuildingGameObj* Defense = (BuildingGameObj*)Find_Base_Defense(!Player->Get_Player_Type());
		return (Defense && !Defense->Is_Destroyed() && Defense->Is_Power_Enabled() && (Player->Get_GameObj()->Get_Vehicle() ? Player->Get_GameObj()->Get_Vehicle()->Get_Is_Scripts_Visible() : Player->Get_GameObj()->Is_Visible()));
	}

	virtual void Activate(cPlayer* Player) {
		DA::Host_Message("A %ls player just got the Spy Crate. Better watch your base %ls!", Get_Wide_Team_Name(Player->Get_Player_Type()), Get_Wide_Team_Name(!Player->Get_Player_Type()));
		SoldierGameObj* Soldier = Player->Get_GameObj();
		ArKaSpyChatObserverClass* SpyChat = new ArKaSpyChatObserverClass;
		SpyChat->Set_Time(300);

			SoldierGameObjDef* SBH = (SoldierGameObjDef*)Find_Named_Definition("CnC_Nod_FlameThrower_2SF");
			if (SBH) {
				Soldier->Re_Init(*SBH);
				Soldier->Post_Re_Init();
			}
			Soldier->Add_Observer(new ArKaSpyObserverClass);
			Soldier->Add_Observer(SpyChat);

			DA::Page_Player(Player, "Base defenses will ignore your current character until you destroy an enemy building.");
			DA::Page_Player(Player, "You can now spy into the enemy's teamchat for 5 minutes.");

	}
};
Register_Crate(ArKaSpyCrateClass, "Spy_And_Chat", DACrateType::INFANTRY);

/*******************************************************************************************************************************/

/********************************************* NSA CRATE ***********************************************************************/


class ArKaNSACrateClass : public DACrateClass {
	
	
	virtual bool Can_Activate(cPlayer* Player) {
		SoldierGameObj* Soldier = Player->Get_GameObj();

		if (!Soldier->Find_Observer("ArKaSpyChatObserverClass") && Player->Get_GameObj()->Is_Visible()) {
			return true;
		}
		else 
			return false;
	}


	virtual void Activate(cPlayer* Player) {
		SoldierGameObj* Soldier = Player->Get_GameObj();
		ArKaSpyChatObserverClass* NSAChat = new ArKaSpyChatObserverClass;
		
		NSAChat->Set_Time(120);
		Soldier->Add_Observer(NSAChat);
		DA::Page_Player(Player, "You can now spy into the enemy's teamchat for 2 minutes by the NSA crate.");
	}
};
Register_Crate(ArKaNSACrateClass, "NSA", DACrateType::INFANTRY);


// --------------------------------------------- DEBUG COMMAND ------------------------------------------------------------------
class ArKa_Debug : public DAChatCommandClass {
	virtual bool Activate(cPlayer* Player, const DATokenClass& Text, TextMessageEnum ChatType) {
		Console_Output("SpyCount: %i\n", SpyChatPlugIn.SpyCount );
		Console_Output("Is Player %i spy: %s\n",Player->Get_Id() , SpyChatPlugIn.SpyList[Player->Get_Id()]?"true":"false");
		Console_Output("MAX_PLAYERS: %i\n", MAX_PLAYERS);

		return false;
	}
};
// Register_Simple_Chat_Command(ArKa_Debug, "!DEBUG");